import asyncio
import enum
import os

import websockets

import mini
from mini import WiFiDevice as _WiFiDevice
from mini.channels import msg_utils
from mini.pb2.pccodemao_message_pb2 import Message as _Message
from mini.pb2.pccodemao_messageheader_pb2 import MessageHeader as _MessageHeader

_found_devices = {}


@enum.unique
class PCPyCmdId(enum.Enum):
    """
    安装程序包
    """
    PYPI_INSTALL_WHEEL_REQUEST = 1

    """
    卸载程序包
    """
    PYPI_UNINSTALL_WHEEL_REQUEST = 2

    """/ **
    *获取程序包信息
    * /"""
    PYPI_GET_WHEEL_INFO_REQUEST = 3

    """/ **
    *执行程序
    * /"""
    PYPI_RUN_WHEEL_REQUEST = 4

    """/ **
    *获取安装程序列表
    * /"""
    PYPI_GET_WHEEL_LIST_REQUEST = 5


def _read_into_buffer(filename) -> bytes:
    buf = bytearray(os.path.getsize(filename))
    with open(filename, 'rb') as f:
        f.readinto(buf)
    return bytes(buf)


async def _send_msg1(websocket, message: _Message) -> str:
    await websocket.send(msg_utils.base64_encode(message.SerializeToString()))
    result: str = ""
    while True:
        try:
            _data = await websocket.recv()
            _bytes = msg_utils.base64_decode(_data)
            msg: _Message = msg_utils.parse_msg(_bytes)
            header: _MessageHeader = msg.header
            if header.command == PCPyCmdId.PYPI_INSTALL_WHEEL_REQUEST.value:
                from mini.tool.pb2.PyPi_InstallWheel_pb2 import InstallWheelResponse
                response: InstallWheelResponse = InstallWheelResponse()
                response.ParseFromString(msg.bodyData)
                print("{0}".format(response.message))
            elif header.command == PCPyCmdId.PYPI_UNINSTALL_WHEEL_REQUEST.value:
                from mini.tool.pb2.PyPi_UninstallWheel_pb2 import UninstallWheelResponse
                response: UninstallWheelResponse = UninstallWheelResponse()
                response.ParseFromString(msg.bodyData)
                print("{0}".format(response.message))
            elif header.command == PCPyCmdId.PYPI_RUN_WHEEL_REQUEST.value:
                from mini.tool.pb2.PyPi_RunWheel_pb2 import RunWheelResponse
                response: RunWheelResponse = RunWheelResponse()
                response.ParseFromString(msg.bodyData)
                print("{0}".format(response.message))
            elif header.command == PCPyCmdId.PYPI_GET_WHEEL_INFO_REQUEST.value:
                from mini.tool.pb2.PyPi_GetWheelInfo_pb2 import GetWheelInfoResponse
                response: GetWheelInfoResponse = GetWheelInfoResponse()
                response.ParseFromString(msg.bodyData)
                return response.message
            elif header.command == PCPyCmdId.PYPI_GET_WHEEL_LIST_REQUEST.value:
                from mini.tool.pb2.PyPi_GetWheelList_pb2 import GetWheelListResponse
                response: GetWheelListResponse = GetWheelListResponse()
                response.ParseFromString(msg.bodyData)
                if response.resultCode != 0:
                    return response.error
                else:
                    return "\n".join(response.Wheels)
            elif header.target == -1:
                print(f"不支持的指令cmd={header.command}")
            else:
                print(f"不支持的指令cmd={header.command}")

        except Exception as e:
            if isinstance(e, websockets.ConnectionClosedOK):
                # print(f"connection closed ok!")
                break
            else:
                raise e
    return result


async def _send_msg0(message: _Message, device: _WiFiDevice) -> str:
    try:
        async with websockets.connect('ws://{}:{!r}'.format(device.address, 8801)) as websocket:
            return await _send_msg1(websocket, message)
    except Exception as e:
        return ""


def _get_eggInfo_path():
    return _get_file('.', 'egg-info', True)


def _get_file(dir_path: str, suffix: str, is_dir: bool = False):
    for temp_path in os.listdir(dir_path):
        temp_path = os.path.join(dir_path, temp_path)
        if is_dir:
            if not os.path.isdir(temp_path):
                continue
        else:
            if not os.path.isfile(temp_path):
                continue

        if temp_path.endswith(suffix):
            result = os.path.abspath(temp_path)
            return result


def _remove_dir(dir_path: str):
    """
    删除目录
    :param dir_path: 需要删除的目录
    """
    if isinstance(dir_path, str) and os.path.isdir(dir_path):
        import shutil
        shutil.rmtree(dir_path)


def setup_py_pkg(project_dir: str) -> str:
    """
    将一个py工程打包成一个.whl文件。
    Args:
        project_dir: 工程文件根目录

    Returns: str, 生成的.whl文件绝对路径
    """
    # 校验目录
    if not os.path.isdir(project_dir):
        print('project_dir must be a directory')
        return ""
    # 将当前进程工作目录切换到工程目录
    os.chdir(project_dir)
    # 清空产物目录
    build_path = 'build'
    egg_info_path = _get_eggInfo_path()
    dist_path = 'dist'
    _remove_dir(build_path)
    _remove_dir(egg_info_path)
    _remove_dir(dist_path)
    # 找到setup文件
    setup_path = 'setup.py'

    if not os.path.isfile(setup_path):
        print('setup.py not exist')
        return ""

    # 打包
    os.system(f"python {setup_path} sdist bdist_wheel")

    # 删除其他临时文件
    egg_info_path = _get_eggInfo_path()
    _remove_dir(build_path)
    _remove_dir(egg_info_path)

    result = _get_file(dist_path, '.whl')
    print(f'result {result}')
    return result


def _build_install_py_pkg_msg(package_path: str, debug: bool) -> _Message:
    from mini.tool.pb2.PyPi_InstallWheel_pb2 import InstallWheelRequest
    request = InstallWheelRequest()
    # 获取文件路径中的文件名
    request.wheelName = os.path.basename(package_path)
    # 把文件转成字节
    request.serializePacket = _read_into_buffer(package_path)
    request.debug = debug
    cmd_id = PCPyCmdId.PYPI_INSTALL_WHEEL_REQUEST.value
    # message
    message: _Message = msg_utils.build_request_msg(cmd_id, 0, request)
    return message


def install_py_pkg(package_path: str, robot_id: str, debug: bool = True):
    """
    将一个py程序安装包,安装到指定序列号的机器人上.
    Args:
        package_path: 安装包的绝对路径
        robot_id:机器人序列号
        debug:是否打印在机器人端卸载pkg时的log

    Returns: None
    """
    # 校验文件
    if not os.path.isfile(package_path):
        print(f'file is not exist:{package_path}')
        return

    base_name = os.path.basename(package_path)

    if not base_name.endswith('.whl'):
        print(f'文件不是一个pypi安装包')
        return

    device: _WiFiDevice = _found_devices.get(robot_id)
    # 搜索设备
    if device is None:
        device = asyncio.run((mini.get_device_by_name(robot_id, 10)))
        if device is None:
            print(f"不能找到机器人:{robot_id}")
            return
        else:
            _found_devices[robot_id] = device
    # 上传
    asyncio.run(_send_msg0(_build_install_py_pkg_msg(package_path, debug), device))


def _build_uninstall_py_pkg_msg(pkg_name: str, debug: bool) -> _Message:
    from mini.tool.pb2.PyPi_UninstallWheel_pb2 import UninstallWheelRequest
    request = UninstallWheelRequest()
    # pkg name
    request.wheelName = os.path.basename(pkg_name)
    request.debug = debug
    cmd_id = PCPyCmdId.PYPI_UNINSTALL_WHEEL_REQUEST.value
    # message
    message: _Message = msg_utils.build_request_msg(cmd_id, 0, request)
    return message


def uninstall_py_pkg(pkg_name: str, robot_id: str, debug: bool = True):
    """
    将一个安装的py程序,从指定序列号的机器人上卸载掉,例如有一个py程序,它的setup.py文件, 配置如下:
    setuptools.setup(
        name="tts_demo",
        ...#省略
    ),
    那么,它的pkg_name为"tts_demo".
    Args:
        pkg_name:程序名称
        robot_id:机器人序列号
        debug:是否打印在机器人端卸载pkg时的log

    Returns:None

    """
    device: _WiFiDevice = _found_devices.get(robot_id)
    # 搜索设备
    if device is None:
        device = asyncio.run((mini.get_device_by_name(robot_id, 10)))
        if device is None:
            print(f"不能找到机器人:{robot_id}")
            return
        else:
            _found_devices[robot_id] = device
    # 卸载
    asyncio.run(_send_msg0(_build_uninstall_py_pkg_msg(pkg_name, debug), device))


def _build_query_py_pkg_msg(pkg_name: str) -> _Message:
    from mini.tool.pb2.PyPi_GetWheelInfo_pb2 import GetWheelInfoRequest
    request = GetWheelInfoRequest()
    # pkg name
    request.wheelName = os.path.basename(pkg_name)
    cmd_id = PCPyCmdId.PYPI_GET_WHEEL_INFO_REQUEST.value
    # message
    message: _Message = msg_utils.build_request_msg(cmd_id, 0, request)
    return message


def query_py_pkg(pkg_name: str, robot_id: str) -> str:
    """
    查询机器人上pkg_name指定的py程序, 其详细信息, 例如有一个py程序,它的setup.py文件, 配置如下:
    setuptools.setup(
        name="tts_demo",
        version="0.0.2",
        author='Gino Deng',
        author_email='jingjing.deng@ubtrobot.com',
        description="demo with mini_sdk",
        long_description='demo with mini_sdk,xxxxxxx',
        long_description_content_type="text/markdown",
        license="GPLv3",
        packages=setuptools.find_packages(),
        classifiers=[
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        install_requires=[
            'alphamini',
        ],
    ),
    查询时, 指定pkg_name="tts_demo", 返回信息如下:
        Name: tts-demo
        Version: 0.0.2
        Summary: demo with mini_sdk
        Home-page: UNKNOWN
        Author: Gino Deng
        Author-email: jingjing.deng@ubtrobot.com
        License: GPLv3
        Location: /data/data/com.termux/files/usr/lib/python3.8/site-packages
        Requires: alphamini
        Required-by:
    Args:
        pkg_name:py程序名,
        robot_id:机器人序列号

    Returns: str
    """
    device: _WiFiDevice = _found_devices.get(robot_id)
    # 搜索设备
    if device is None:
        device = asyncio.run((mini.get_device_by_name(robot_id, 10)))
        if device is None:
            print(f"不能找到机器人:{robot_id}")
            return ""
        else:
            _found_devices[robot_id] = device
    # 查询
    return asyncio.run(_send_msg0(_build_query_py_pkg_msg(pkg_name), device))


def show_py_pkg(pkg_name: str, robot_id: str):
    print(f'{query_py_pkg(pkg_name, robot_id)}')


def _build_list_py_pkg_msg():
    from mini.tool.pb2.PyPi_GetWheelList_pb2 import GetWheelListRequest
    request = GetWheelListRequest()
    cmd_id = PCPyCmdId.PYPI_GET_WHEEL_LIST_REQUEST.value
    # message
    message: _Message = msg_utils.build_request_msg(cmd_id, 0, request)
    return message


def list_py_pkg(robot_id: str) -> str:
    """
    列出机器人上安装的py程序, 例如:
        Package    Version
        ---------- -------
        alphamini  0.1.0
        ifaddr     0.1.7
        pip        20.1.1
        protobuf   3.12.2
        setuptools 47.3.1
        six        1.15.0
        tts-demo   0.0.2
        websockets 8.1
        wheel      0.34.2
    Args:
        robot_id: 机器人序列号

    Returns: str
    """
    device: _WiFiDevice = _found_devices.get(robot_id)
    # 搜索设备
    if device is None:
        device = asyncio.run((mini.get_device_by_name(robot_id, 10)))
        if device is None:
            print(f"不能找到机器人:{robot_id}")
            return ""
        else:
            _found_devices[robot_id] = device
    # 查询
    return asyncio.run(_send_msg0(_build_list_py_pkg_msg(), device))


def list_py_pkg2(robot_id: str):
    print(f'{list_py_pkg(robot_id)}')


def _build_run_py_pkg_msg(entry_point: str, debug: bool) -> _Message:
    from mini.tool.pb2.PyPi_RunWheel_pb2 import RunWheelRequest
    request = RunWheelRequest()
    # pkg main entry
    request.wheelName = entry_point
    request.debug = debug
    cmd_id = PCPyCmdId.PYPI_RUN_WHEEL_REQUEST.value
    # message
    message: _Message = msg_utils.build_request_msg(cmd_id, 0, request)
    return message


def run_py_pkg(entry_point: str, robot_id: str, debug: bool = True):
    """
    触发一个指定的py程序, 在指定的机器人上运行
    Args:
        entry_point: py程序console scripts名称, 例如在setup.py文件中,如下配置:
        entry_points={
            'console_scripts': [
                'XXX = Packages.Modules:XXX'
            ],
        }, 程序入口名称为xxx
        robot_id: 机器人序列号
        debug: 是否回传程序执行时日志

    Returns: None
    """
    device: _WiFiDevice = _found_devices.get(robot_id)
    # 搜索设备
    if device is None:
        device = asyncio.run((mini.get_device_by_name(robot_id, 10)))
        if device is None:
            print(f"不能找到机器人:{robot_id}")
            return
        else:
            _found_devices[robot_id] = device
    # 触发
    asyncio.run(_send_msg0(_build_run_py_pkg_msg(entry_point, debug), device))



#!/usr/bin/env python3

import enum

from ..apis.base_api import BaseApi, DEFAULT_TIMEOUT
from ..apis.cmdid import PCProgramCmdId
from ..pb2 import cloudstorageurls_pb2
from ..pb2.cloudtranslate_pb2 import Platform
from ..pb2.codemao_controltts_pb2 import ControlTTSRequest, ControlTTSResponse
from ..pb2.codemao_getaudiolist_pb2 import GetAudioListRequest, GetAudioListResponse
from ..pb2.codemao_playaudio_pb2 import PlayAudioRequest, PlayAudioResponse
from ..pb2.codemao_playonlinemusic_pb2 import MusicRequest, MusicResponse
from ..pb2.codemao_stopaudio_pb2 import StopAudioRequest, StopAudioResponse
from ..pb2.pccodemao_message_pb2 import Message


@enum.unique
class TTSControlType(enum.Enum):
    """
    TTS控制类型

    播放/停止
    """
    START = 1  # 播放
    STOP = 0  # 停止


class PlayTTS(BaseApi):
    """播放/停止TTS api

    机器人播放合成的TTS语音

    control_type为STOP时，表示停止所有正在播放的TTS

    Args:
        is_serial (bool): 是否等待回复，默认True
        text (str): 播放的文本，不能为空或者None
        control_type (TTSControlType): 控制类型，默认START，开始播放
    """

    def __init__(self, is_serial: bool = True, text: str = None, control_type: TTSControlType = TTSControlType.START):
        assert text is not None and len(text), 'tts text should be available'
        self.__isSerial = is_serial
        self.__text = text
        self.__type = control_type.value

    async def execute(self):
        """
        执行控制TTS指令

        Returns:
            ControlTTSResponse:

        """
        timeout = 0
        if self.__isSerial:
            timeout = DEFAULT_TIMEOUT

        request = ControlTTSRequest()
        request.text = self.__text
        request.type = self.__type

        cmd_id = PCProgramCmdId.PLAY_TTS_REQUEST.value

        return await self.send(cmd_id, request, timeout)

    def parse_msg(self, message):
        """

        Args:
            message (Message):待解析的Message对象

        Returns:
            ControlTTSResponse:

        """
        if isinstance(message, Message):
            data = message.bodyData
            response = ControlTTSResponse()
            response.ParseFromString(data)
            return response
        else:
            return None


@enum.unique
class AudioStorageType(enum.Enum):
    """
    音频存储类型

    阿里私有云/公网/机器人本地内置/机器人本地自定义

    """
    ALIYUN_PRIVATE = 1  # 阿里私有云
    NET_PUBLIC = 2  # 公网
    PRESET_LOCAL = 3  # 本地内置
    CUSTOMIZE_LOCAL = 4  # 本地自定义


class PlayAudio(BaseApi):
    """播放音频api

    机器人播放指定的音频

    Args:
        is_serial (bool): 是否等待回复，默认True
        url (str): 音频地址，当storage_type为ALIYUN_PRIVATE/NET_PUBLIC，url为音频文件网址；当storage_type为PRESET_LOCAL/CUSTOMIZE_LOCAL时，url为本地音频名称（本地音频名称可通过FetchAudioList接口获取）
        storage_type (AudioStorageType): 音频存储类型，默认ALIYUN_PRIVATE，阿里私有云
        volume (float): 音量大小，范围[0.0,1.0]，默认1.0

    """

    def __init__(self, is_serial: bool = True, url: str = None,
                 storage_type: AudioStorageType = AudioStorageType.ALIYUN_PRIVATE, volume: float = 1.0):

        assert url is not None and len(url), 'PlayAudio url should be available'
        assert 0 <= volume <= 1.0, 'PlayAudio volume should be in range[0,1]'
        self.__isSerial = is_serial
        self.__url = url
        self.__volume = volume
        self.__cloudStorageType = storage_type.value

    async def execute(self):
        """
        执行播放音频指令

        Returns:
            PlayAudioResponse:

        """
        timeout = 0
        if self.__isSerial:
            timeout = DEFAULT_TIMEOUT

        cloud = cloudstorageurls_pb2.CloudStorage()
        cloud.type = self.__cloudStorageType
        cloud.url.extend([self.__url])

        request = PlayAudioRequest()

        request.cloud.CopyFrom(cloud)
        request.volume = self.__volume

        cmd_id = PCProgramCmdId.PLAY_AUDIO_REQUEST.value

        return await self.send(cmd_id, request, timeout)

    def parse_msg(self, message):
        """

        Args:
            message (Message):待解析的Message对象

        Returns:
            PlayAudioResponse:

        """
        if isinstance(message, Message):
            data = message.bodyData
            response = PlayAudioResponse()
            response.ParseFromString(data)
            return response
        else:
            return None


class StopAllAudio(BaseApi):
    """停止所有音频api

    机器人停止所有正在播放的音频

    Args:
        is_serial (bool): 是否等待回复，默认True

    """

    def __init__(self, is_serial: bool = True):
        self.__isSerial = is_serial

    async def execute(self):
        """
        执行停止所有音频指令

        Returns:
            StopAudioResponse:

        """
        timeout = 0
        if self.__isSerial:
            timeout = DEFAULT_TIMEOUT

        request = StopAudioRequest()

        cmd_id = PCProgramCmdId.STOP_AUDIO_REQUEST.value

        return await self.send(cmd_id, request, timeout)

    def parse_msg(self, message):
        """

        Args:
            message (Message):待解析的Message对象

        Returns:
            StopAudioResponse:

        """
        if isinstance(message, Message):
            data = message.bodyData
            response = StopAudioResponse()
            response.ParseFromString(data)
            return response
        else:
            return None


@enum.unique
class AudioSearchType(enum.Enum):
    """
    音频查询类型

    内置/自定义
    """
    INNER = 0  # 内置
    CUSTOM = 1  # 自定义


class FetchAudioList(BaseApi):
    """获取机器人的音频列表api

    获取存储在机器人rom或者sdcard中的音频列表

    Args:
        is_serial (bool): 是否等待回复，默认True
        search_type (AudioSearchType): 查询类型，默认INNER，即机器人内置;CUSTOM表示自定义，存放在sdcard中
    """

    def __init__(self, is_serial: bool = True, search_type: AudioSearchType = AudioSearchType.INNER):
        self.__isSerial = is_serial
        self.__searchType = search_type.value

    async def execute(self):
        """
        执行获取音频列表指令

        Returns:
            GetAudioListResponse:

        """
        timeout = 0
        if self.__isSerial:
            timeout = DEFAULT_TIMEOUT

        request = GetAudioListRequest()

        request.searchType = self.__searchType

        cmd_id = PCProgramCmdId.GET_AUDIO_LIST_REQUEST.value
        return await self.send(cmd_id, request, timeout)

    def parse_msg(self, message):
        """

        Args:
            message (Message):待解析的Message对象

        Returns:
            GetAudioListResponse:

        """
        if isinstance(message, Message):
            data = message.bodyData
            response = GetAudioListResponse()
            response.ParseFromString(data)
            return response
        else:
            return None


class PlayOnlineMusic(BaseApi):
    """播放在线歌曲api

    机器人播放QQ音乐在线歌曲，需在app绑定机器人并授权

    Args:
        is_serial (bool): 是否等待回复，默认True
        name (str): 歌曲名称，不能为空或者None

    """

    def __init__(self, is_serial: bool = True, name: str = None):
        assert name is not None and len(name), 'PlayOnlineMusic name should be available'
        self.__isSerial = is_serial
        self.__name = name
        self.__platform = Platform.TENCENT

    async def execute(self):
        """
        执行播放在线歌曲指令

        Returns:
            MusicResponse:

        """
        timeout = 0
        if self.__isSerial:
            timeout = DEFAULT_TIMEOUT

        request = MusicRequest()

        request.platform = self.__platform
        request.name = self.__name

        cmd_id = PCProgramCmdId.PLAY_ONLINE_MUSIC_REQUEST.value
        return await self.send(cmd_id, request, timeout)

    def parse_msg(self, message):
        """

        Args:
            message (Message):待解析的Message对象

        Returns:
            MusicResponse:

        """
        if isinstance(message, Message):
            data = message.bodyData
            response = MusicResponse()
            response.ParseFromString(data)
            return response
        else:
            return None



#!/usr/bin/env python3
import abc
import asyncio
import enum
from abc import ABC
from typing import Callable, Union

from ..channels.websocket_client import ubt_websocket as _UBTWebSocket, AbstractMsgHandler

DEFAULT_TIMEOUT = 300

socket = _UBTWebSocket()


@enum.unique
class MiniApiResultType(enum.Enum):
    """
    Api的返回结果状态类型
    """
    Success = 1  # 成功
    Timeout = 2  # 超时
    Unsupported = 3  # 不支持的命令


class BaseApi(abc.ABC):
    """消息api基类
    """

    async def send(self, cmd_id: int, message, timeout: int) -> Union[object, bool]:
        """发送消息方法

        由子类函数内部调用，子类实例不可调用。

        Args:
            cmd_id (int): 支持的命令id，例如：mini.apis.cmdid.PLAY_ACTION_REQUEST
            message (Message): 支持的消息实体，例如：mini.pb2.PlayActionRequest
            timeout (int): 超时时间，当timeout<=0时，表示不需要等待机器人回复，当timeout>0时，表示需要等待机器人回复，

        Returns:
            当timeout<=0时，返回True/False，表示消息是否发送成功
            当timeout>0时，
            如果消息超时或者发送失败，则返回tuple(MiniApiResultType.Timeout,None)
            如果消息有回复，则返回tuple(MiniApiResultType.Success,result),result为相应的回复消息

        """
        assert cmd_id >= 0, 'cmdId should not be negative number in BaseApi'
        assert message is not None, 'message should not be none in BaseApi'
        # 通用的发送消息逻辑
        if timeout <= 0:
            return await socket.send_msg0(cmd_id, message)
        else:
            result = await socket.send_msg(cmd_id, message, timeout)
            if result:
                if message.header.target == -1:
                    import logging
                    log = logging.getLogger(__name__)
                    log.addHandler(logging.StreamHandler())
                    log.setLevel(logging.WARNING)
                    log.warning(f'当前机器人版本不支持命令:cmd={message.header.command}, 请升级机器人系统版本.')
                    return MiniApiResultType.Unsupported, None
                else:
                    return MiniApiResultType.Success, self.parse_msg(result)
            else:
                return MiniApiResultType.Timeout, None

    async def execute(self):
        """发送指令

        将支持的message序列化后,写入socket
        由子类实现
        """
        raise NotImplementedError()

    def parse_msg(self, message):
        """解析回复指令

        将收到的Message对象包含的消息数据反序列化为相应的response
        由子类实现，仅供子类内部调用
        Args:
            message: Message对象

        """
        raise NotImplementedError()


class BaseApiNeedResponse(BaseApi, abc.ABC):
    """
    需要回复的消息api基类，timeout不能为空
    """

    async def send(self, cmd_id, data, timeout: int):
        """重写父类方法

        校验timeout，必须>0
        """

        assert timeout > 0, 'timeout should be Positive number in BaseApiNeedResponse'
        return await super().send(cmd_id, data, timeout)


class BaseApiNoNeedResponse(BaseApi, ABC):
    """不需要回复的消息api基类
    """

    async def send(self, cmd_id, message, timeout: int = 0):
        """重写父类方法

        将timeout设置为0
        """

        return await super().send(cmd_id, message, 0)


class BaseEventApi(BaseApiNoNeedResponse, AbstractMsgHandler, ABC):
    """事件类消息api基类

    当注册了事件处理器后，由机器人主动推送过来的事件消息

    Args:
        cmd_id (int): 注册的指令id
        message (Message): 向机器人发送的注册消息
        is_repeat (bool): 是否重复监听，默认为True
        timeout (int): 超时时间，默认为0
        handler (Callable): 事件消息处理器,f(message)

    """

    def __init__(self, cmd_id: int, message, is_repeat: bool = True, timeout: int = 0,
                 handler: 'Callable[..., None]' = None):
        """初始化事件类消息
        """
        super().__init__()
        self.__cmdId = cmd_id
        self.__request = message
        self.__isRepeat = is_repeat
        self.__timeout = timeout
        self.__handler = handler

        if is_repeat:
            self.__repeatCount = -1
        else:
            self.__repeatCount = 1

    def set_handler(self, handler: 'Callable[..., None]' = None):
        """设置事件消息处理器

        Args:
            handler (Callable): 事件消息处理器，f(message)

        """
        self.__handler = handler

    def start(self):
        """启动监听器
        """

        # 发送消息
        asyncio.create_task(self.send(cmd_id=self.__cmdId, message=self.__request))
        # 注册监听
        socket.register_msg_handler(cmd=self.__cmdId, handler=self)

    def stop(self):
        """停止监听器

        子类需要通知机器人停止事件上报
        """

        # 移除消息监听
        socket.unregister_msg_handler(cmd=self.__cmdId, handler=self)

    # AbstractMsgHandler
    def handle_msg(self, message):
        # 处理监听次数
        if self.__repeatCount > 0:
            # 有监听次数
            self.__handle_msg(message)
            self.__repeatCount -= 1
        elif self.__repeatCount == -1:
            # 无限监听
            self.__handle_msg(message)

    def __handle_msg(self, message):
        if message.header.target == -1:
            import logging
            log = logging.getLogger(__name__)
            log.addHandler(logging.StreamHandler())
            log.setLevel(logging.WARNING)
            log.warning(f'当前机器人版本不支持命令:cmd={message.header.command}, 请升级机器人系统版本.')
            return
        if self.__handler is not None:
            self.__handler(self.parse_msg(message))
